//
//  PersonTableController.m
//  Person Database
//
//  Created by Holger Hinzberg on 19.02.12.
//  Copyright (c) 2012 Cocoa-Coding. All rights reserved.
//

#import "PersonTableController.h"

// Ein Define für das Drag Item, 
// dann geht‘s später einfacher.
#define TableDragItem @"Cocoapraxis.PersonDB.TableItem"

@implementation PersonTableController
@synthesize personTable = _personTable;

#pragma mark -
#pragma mark Initialisierung

- (id)init
{
    if ((self = [super init]))
    {
        personArray = [[NSMutableArray alloc] init];
        isSortOrderAscending = YES;
        lastClickedIdentifier = @"";
        lastClickedColumn = nil;
    }
    return self;
}

- (void)awakeFromNib
{
    // Den Typ für die Tabelle registrieren.
    [_personTable registerForDraggedTypes:
     [NSArray arrayWithObject:TableDragItem]];
}

- (void)addPerson:(Person *)aPerson
{
    // Opbjekt dem Array hinzufügen
    [personArray addObject:aPerson];
    // Tabellen-Steuerelement aktualisieren
    [_personTable reloadData];
}

#pragma mark -
#pragma mark Datasource & Delegate

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [personArray count];
}
 
- (id)tableView:(NSTableView *)aTableView
objectValueForTableColumn:
(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
    // Das Objekt für eine Zeile
    Person *aPerson = [personArray objectAtIndex:rowIndex];
    
    // Die Eigenschaft für die Spalte,
    // ermittelt mit dem Identifier
    id value =
    [aPerson valueForKey:[aTableColumn identifier]];
    return value;
}

-(void)tableView:(NSTableView*)tableView 
didClickTableColumn:(NSTableColumn *)tableColumn
{
    // Den Identifier ermitteln.
    NSString * ident = tableColumn.identifier;
    
    if ([ident isEqualToString:lastClickedIdentifier])
    {
        NSLog(@"Gleiche Spalte angeklickt");
        // Den Wert umkehren
        isSortOrderAscending = !isSortOrderAscending;
    }
    else
    {
        NSLog(@"Andere Spalte angeklickt");
        // Den Wert zurücksetzen
        isSortOrderAscending = YES;
        // Diesen Identifier jetzt speichern
        lastClickedIdentifier = ident;
    }

    
    // Sortierungen nach Eigenschaften.
    if([ident isEqualToString:@"name"])
    {
        [personArray sortUsingSelector:
         @selector(compareWithName:)];
    }
    else if([ident isEqualToString:@"firstname"])
    {
        [personArray sortUsingSelector:
         @selector(compareWithFirstname:)];
    }
    else if([ident isEqualToString:@"age"])
    {
        [personArray sortUsingSelector:
         @selector(compareWithAge:)];
    }
    
    
    if(isSortOrderAscending == NO)
    {
        int y = 0;
        int x = personArray.count - 1;
        while (y < x)
        {
            // Objekte im Array tauschen
            [personArray exchangeObjectAtIndex:y
                             withObjectAtIndex:x];
            y++;
            x--;
        }
    }
    
    
    // Pfeile zur Sortierung zeichnen.
    [self drawSortArrows:isSortOrderAscending
               forColumn:tableColumn];
    
    // Tabellen-Steuerelement aktualisieren
    [_personTable reloadData];
}

- (void)drawSortArrows:(BOOL)isSortAscending 
             forColumn:(NSTableColumn *)tableColumn
{
    // Ist die Spalte unterschiedlich, 
    // muss das Bild der alten Spalte entfernt werden.
    if (lastClickedColumn != tableColumn)
    {
        [_personTable setIndicatorImage: nil
                         inTableColumn: lastClickedColumn];
        lastClickedColumn = tableColumn;
    }
    
    // Ein Zeiger auf ein Symbol.
    NSImage *indicatorImage;
    
    // Passende Bilder aus dem
    // Framework laden.
    if(isSortAscending == YES)
    {
        indicatorImage =
        [NSImage imageNamed:
         @"NSAscendingSortIndicator"];
    }
    else
    {
        indicatorImage =
        [NSImage imageNamed:
         @"NSDescendingSortIndicator"];
    }
    
    // Das Symbol für Tabelle und Spalte setzen.
    [_personTable setIndicatorImage: 
     indicatorImage inTableColumn: tableColumn];
}

#pragma mark -
#pragma mark Drag & Drop 


- (BOOL)tableView:(NSTableView *)tv 
writeRowsWithIndexes:(NSIndexSet *)rowIndexes
     toPasteboard:(NSPasteboard*)pboard
{
    NSLog(@"Drag beginnt!");
    
    // Den Row Index in NSData umwandeln.
    NSData *data =
    [NSKeyedArchiver archivedDataWithRootObject
     :rowIndexes];
    
    // Der Zwischenablage unseren selbst
    // definieren Typ hinzufügen.
    [pboard declareTypes:
     [NSArray arrayWithObject:TableDragItem]
                   owner:self];
    
    // Daten mit passendem Typ in die 
    // Zwischenablage schreiben.
    [pboard setData:data forType:TableDragItem];
    
    return YES;
}

- (NSDragOperation)tableView:(NSTableView*)tv 
                validateDrop:(id <NSDraggingInfo>)info
                 proposedRow:(NSInteger)row 
       proposedDropOperation:(NSTableViewDropOperation)op
{
    NSLog(@"Drop überprüfen.");
    
    // Mauspfeil ist zwischen den Zeilen.
    // Hier darf abgelegt werden.
    if (op == NSTableViewDropAbove)
    {
        NSLog(@"Gültige Position");
        return NSDragOperationMove;
    }
    
    NSLog(@"Ungültige Position");
    
    return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView *)aTableView 
       acceptDrop:(id <NSDraggingInfo>)info
              row:(NSInteger)row 
    dropOperation:(NSTableViewDropOperation)operation
{
    NSLog(@"Drop - Abgelegt!");
    
    // NSData wieder ein NSIndexSet umwandeln.
    NSPasteboard* pboard = [info draggingPasteboard];
    NSData* rowData = [pboard dataForType:TableDragItem];
    
    NSIndexSet* rowIndexes =
    [NSKeyedUnarchiver
     unarchiveObjectWithData:rowData];
    
    NSUInteger insertAboveIndexCount = 0;
    NSUInteger removeIndex = 0;
    // Den ersten Index aus dem IndexSet holen
    NSUInteger draggedRow = [rowIndexes firstIndex];
    
    while(draggedRow != NSNotFound)
    {
        if(draggedRow >= row)
        {
            removeIndex =
            draggedRow + insertAboveIndexCount;
            insertAboveIndexCount++;
        }
        else
        {
            removeIndex = draggedRow;
            row--;
        }
        
        // Objekt holen
        Person *aPerson = 
        [personArray objectAtIndex:removeIndex];
        
        // Objekt entfernen
        [personArray removeObjectAtIndex:removeIndex];
        
        // Objekt einfügen
        [personArray insertObject:aPerson atIndex:row];
        
        // Den nächsten Index holen
        draggedRow = [rowIndexes indexLessThanIndex:draggedRow];
    }
    
    // Tabelle neu laden
    [aTableView reloadData];
    
    // Alles OK:
    return YES;

}

@end
